@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2009 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2010 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -38,6 +38,7 @@
 #include "http.h"
 #include "validator.h"
 #include "nullable.h"
+#include "encoder.h"
 
 #define CHEROKEE_CONFIG_PRIORITY_NONE    0
 #define CHEROKEE_CONFIG_PRIORITY_DEFAULT 1
@@ -49,6 +50,24 @@ typedef enum {
 	cherokee_expiration_time
 } cherokee_expiration_t;
 
+typedef enum {
+	cherokee_expiration_prop_none              = 0,
+
+	/* cacheable */
+	cherokee_expiration_prop_public            = 1,
+	cherokee_expiration_prop_private           = 1 << 1,
+	cherokee_expiration_prop_no_cache          = 1 << 2,
+
+	/* stored */
+	cherokee_expiration_prop_no_store          = 1 << 3,
+	cherokee_expiration_prop_no_transform      = 1 << 4,
+
+	/* other props */
+	cherokee_expiration_prop_must_revalidate   = 1 << 5,
+	cherokee_expiration_prop_proxy_revalidate  = 1 << 6,
+} cherokee_expiration_props_t;
+
+
 typedef struct {
 	/* Properties
 	 */
@@ -56,6 +75,7 @@ typedef struct {
 	cherokee_boolean_t          only_secure;
 	cherokee_null_bool_t        no_log;
 	void                       *access;
+	cherokee_list_t            *header_ops;
 
 	/* Handler
 	 */
@@ -76,6 +96,7 @@ typedef struct {
 	 */
 	cherokee_expiration_t       expiration;
 	time_t                      expiration_time;
+	cherokee_expiration_props_t expiration_prop;
 
 	/* Encoding
 	 */
@@ -99,9 +120,16 @@ ret_t cherokee_config_entry_free     (cherokee_config_entry_t  *entry);
 ret_t cherokee_config_entry_init     (cherokee_config_entry_t  *entry);
 ret_t cherokee_config_entry_mrproper (cherokee_config_entry_t  *entry);
 
-ret_t cherokee_config_entry_add_encoder (cherokee_config_entry_t *entry, cherokee_buffer_t *name, cherokee_plugin_info_t *plugin_info);
-ret_t cherokee_config_entry_set_handler (cherokee_config_entry_t *entry, cherokee_plugin_info_handler_t *plugin_info);
-ret_t cherokee_config_entry_complete    (cherokee_config_entry_t *entry, cherokee_config_entry_t *source);
+ret_t cherokee_config_entry_set_encoder (cherokee_config_entry_t  *entry,
+					 cherokee_buffer_t        *encoder_name,
+					 cherokee_plugin_info_t   *plugin_info,
+					 cherokee_encoder_props_t *encoder_props);
+
+ret_t cherokee_config_entry_set_handler (cherokee_config_entry_t        *entry,
+					 cherokee_plugin_info_handler_t *plugin_info);
+
+ret_t cherokee_config_entry_complete    (cherokee_config_entry_t *entry,
+					 cherokee_config_entry_t *source);
 
 ret_t cherokee_config_entry_print       (cherokee_config_entry_t *entry);
 
