@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2009 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2010 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -43,33 +43,49 @@ typedef enum {
 } cherokee_http_version_t;
 
 typedef enum {
-	http_unknown      = 0,
-	http_all_methods  = 0xFFFFFFF,
+	http_unknown      = 0LL,
+	http_all_methods  = 0x1FFFFFFFFLL,
 
-	http_get          = 1,
-	http_post         = 1 << 1,
-	http_head         = 1 << 2,
-	http_put          = 1 << 3,
-	http_options      = 1 << 4,
-	http_delete       = 1 << 5,
-	http_trace        = 1 << 6,
-	http_connect      = 1 << 7,
+	http_get              = 1,
+	http_post             = 1LL << 1,
+	http_head             = 1LL << 2,
+	http_put              = 1LL << 3,
+	http_options          = 1LL << 4,
+	http_delete           = 1LL << 5,
+	http_trace            = 1LL << 6,
+	http_connect          = 1LL << 7,
 
-	http_copy         = 1 << 8,
-	http_lock         = 1 << 9,
-	http_mkcol        = 1 << 10,
-	http_move         = 1 << 11,
-	http_notify       = 1 << 12,
-	http_poll         = 1 << 13,
-	http_propfind     = 1 << 14,
-	http_proppatch    = 1 << 15,
-	http_search       = 1 << 16,
-	http_subscribe    = 1 << 17,
-	http_unlock       = 1 << 18,
-	http_unsubscribe  = 1 << 19,
-	http_report       = 1 << 20
+	http_copy             = 1LL << 8,
+	http_lock             = 1LL << 9,
+	http_mkcol            = 1LL << 10,
+	http_move             = 1LL << 11,
+	http_notify           = 1LL << 12,
+	http_poll             = 1LL << 13,
+	http_propfind         = 1LL << 14,
+	http_proppatch        = 1LL << 15,
+	http_search           = 1LL << 16,
+	http_subscribe        = 1LL << 17,
+	http_unlock           = 1LL << 18,
+	http_unsubscribe      = 1LL << 19,
+	http_report           = 1LL << 20,
+	http_patch            = 1LL << 21,
+	http_version_control  = 1LL << 22,
+	http_checkout         = 1LL << 23,
+	http_uncheckout       = 1LL << 24,
+	http_checkin          = 1LL << 25,
+	http_update           = 1LL << 26,
+	http_label            = 1LL << 27,
+	http_mkworkspace      = 1LL << 28,
+	http_mkactivity       = 1LL << 29,
+	http_baseline_control = 1LL << 30,
+	http_merge            = 1LL << 31,
+	http_invalid          = 1LL << 32
 } cherokee_http_method_t;
 
+#define cherokee_http_method_LENGTH 33
+#define HTTP_METHOD(x) ((cherokee_http_method_t)(x))
+
+
 typedef enum {
 	http_auth_nothing = 0,
 	http_auth_basic   = 1,
@@ -124,14 +140,18 @@ typedef enum {                               /* Protocol   RFC  Section */
 	http_unprocessable_entity     = 422, /*   WebDAV  2518  10.3    */
 	http_locked                   = 423, /*   WebDAV  2518  10.4    */
 	http_failed_dependency        = 424, /*   WebDAV  2518  10.5    */
+	http_unordered_collection     = 425, /*   WebDAV  3648          */
 	http_upgrade_required         = 426, /* TLS upgr  2817   6      */
+	http_retry_with               = 449, /* Microsoft extension     */
 	http_internal_error           = 500, /* HTTP/1.1  2616  10.5.1  */
 	http_not_implemented          = 501, /* HTTP/1.1  2616  10.5.2  */
 	http_bad_gateway              = 502, /* HTTP/1.1  2616  10.5.3  */
 	http_service_unavailable      = 503, /* HTTP/1.1  2616  10.5.4  */
 	http_gateway_timeout          = 504, /* HTTP/1.1  2616  10.5.5  */
 	http_version_not_supported    = 505, /* HTTP/1.1  2616  10.5.6  */
+	http_variant_also_negotiates  = 506, /* HTTP/1.1  2295   8.1    */
 	http_insufficient_storage     = 507, /* HTTP/1.1  2616  10.6    */
+	http_bandwidth_limit_exceeded = 509, /* Apache extension        */
 	http_not_extended             = 510  /* HTTP Ext  2774   7      */
 } cherokee_http_t;
 
@@ -175,21 +195,25 @@ typedef enum {                               /* Protocol   RFC  Section */
 #define http_unprocessable_entity_string     "422 Unprocessable Entity"
 #define http_locked_string                   "423 Locked"
 #define http_failed_dependency_string        "424 Failed Dependency"
+#define http_unordered_collection_string     "425 Unordered Collection"
 #define http_upgrade_required_string         "426 Upgrade Required"
+#define http_retry_with_string               "449 Retry With"
 #define http_internal_error_string           "500 Internal Server Error"
 #define http_not_implemented_string          "501 Not Implemented"
 #define http_bad_gateway_string              "502 Bad gateway"
 #define http_service_unavailable_string      "503 Service Unavailable"
 #define http_gateway_timeout_string          "504 Gateway Timeout"
 #define http_version_not_supported_string    "505 HTTP Version Not Supported"
+#define http_variant_also_negotiates_string  "506 Variant Also Negotiates"
 #define http_insufficient_storage_string     "507 Insufficient Storage"
+#define http_bandwidth_limit_exceeded_string "509 Bandwidth Limit Exceeded"
 #define http_not_extended_string             "510 Not Extended"
 
 #define http_type_100_max 102
-#define http_type_200_max 206
-#define http_type_300_max 304
-#define http_type_400_max 426
-#define http_type_500_max 505
+#define http_type_200_max 207
+#define http_type_300_max 307
+#define http_type_400_max 449
+#define http_type_500_max 510
 
 #define http_type_100(c)  ((c >= 100) && (c <= http_type_100_max))
 #define http_type_200(c)  ((c >= 200) && (c <= http_type_200_max))
@@ -199,20 +223,28 @@ typedef enum {                               /* Protocol   RFC  Section */
 
 #define http_method_with_body(m)  ((m) != http_head)
 
-#define http_method_with_input(m) ((m == http_post)     || \
-				   (m == http_put)      || \
-				   (m == http_mkcol)    || \
-				   (m == http_search)   || \
-				   (m == http_report)   || \
-				   (m == http_options)  || \
-				   (m == http_propfind) || \
-				   (m == http_proppatch))
+#define http_method_with_input(m) ((m == http_post)      || \
+				   (m == http_put)       || \
+				   (m == http_mkcol)     || \
+				   (m == http_merge)     || \
+				   (m == http_search)    || \
+				   (m == http_report)    || \
+				   (m == http_checkout)  || \
+				   (m == http_propfind)  || \
+				   (m == http_proppatch) || \
+				   (m == http_mkactivity))
+
+#define http_method_with_optional_input(m) (m == http_options)
+
 
 /* RFC 2616: Section 4.3 */
 #define http_code_with_body(e)    ((! http_type_100(e))            /* 1xx */ && \
 				   ((e) != http_no_content)        /* 204 */ && \
 				   ((e) != http_not_modified))     /* 304 */
 
+#define http_port_is_standard(port,is_tls)  (((! is_tls) && (port == 80)) || \
+					     ((  is_tls) && (port == 443)))
+
 ret_t cherokee_http_method_to_string  (cherokee_http_method_t  method,  const char **str, cuint_t *str_len);
 ret_t cherokee_http_string_to_method  (cherokee_buffer_t *string, cherokee_http_method_t *method);
 ret_t cherokee_http_version_to_string (cherokee_http_version_t version, const char **str, cuint_t *str_len);
