@@ -27,7 +27,8 @@ import copy
 
 import CTK
 import Cherokee
-import configured
+
+from configured import *
 
 SAVED_NOTICE     = N_("The Configuration has been Saved")
 SAVED_RESTART    = N_("Would you like to apply the changes to the running server now?")
@@ -38,6 +39,17 @@ URL_SAVE_GRACEFUL = r'/save/apply/graceful'
 URL_SAVE_HARD     = r'/save/apply/hard'
 URL_SAVE_NONE     = r'/save/apply/none'
 
+HELP_HTML = """
+   <div id="help">
+     <a class="helpbutton" href="#" id="help-a"><span>%(help)s</span></a>
+     %(helps)s
+   </div>
+"""
+
+HELP_JS = """
+   $('#help-a').click (function(){ toggleHelp(); });
+"""
+
 SAVE_BUTTON = """
 $('#save-button').bind ('click', function(){
   /* Do nothing if it hasn't changed */
@@ -45,7 +57,7 @@ $('#save-button').bind ('click', function(){
      return;
   }
 
-  %s
+  %(show_dialog_js)s
 })"""
 
 
@@ -88,15 +100,15 @@ class Save:
             submit.bind ('submit_success', dialog.JS_to_close())
             all += submit
         else:
-            # Prompt about the reset
+            # Close the dialog
             all += CTK.RawHTML ('<p>%s</p>' %(_(SAVED_NO_RUNNING)))
 
             submit = CTK.Submitter (URL_SAVE_NONE)
-            submit.id = 'saved-no-running'
             submit += CTK.Hidden('none', 'foo')
-            submit += CTK.SubmitterButton (_('OK'))
             submit.bind ('submit_success', dialog.JS_to_close())
+
             all += submit
+            all += CTK.RawHTML (js = submit.JS_to_submit())
 
         render = all.Render()
         return render.toStr()
@@ -110,21 +122,30 @@ class Base (CTK.Page):
 
         # Set up the template
         template = CTK.Template (filename = theme_file)
-        template['title']    = title
-        template['save']     = _('Save')
-        template['home']     = _('Home')
-        template['status']   = _('Status')
-        template['general']  = _('General')
-        template['vservers'] = _('vServers')
-        template['sources']  = _('Sources')
-        template['advanced'] = _('Advanced')
-        template['help']     = _('Help')
-        template['updating'] = _('Updating...')
+        template['title']     = title
+        template['save']      = _('Save')
+        template['home']      = _('Home')
+        template['status']    = _('Status')
+        template['market']    = _('Market')
+        template['general']   = _('General')
+        template['vservers']  = _('vServers')
+        template['sources']   = _('Sources')
+        template['advanced']  = _('Advanced')
+        template['help']      = _('Help')
+        template['updating']  = _('Updating...')
+        template['HELP_HTML'] = kwargs.has_key('helps') and HELP_HTML or ''
+        template['HELP_JS']   = kwargs.has_key('helps') and HELP_JS   or ''
 
         # <body> property
         if body_id:
             template['body_props'] = ' id="body-%s"' %(body_id)
 
+        # Hide/Show Market icon
+        if int (CTK.cfg.get_val("admin!ows!enabled", OWS_ENABLE)):
+            template['market_menu_entry'] = '<li id="nav-market"><a href="/market">%(market)s</a></li>'
+        else:
+            template['market_menu_entry'] = ''
+
         # Save dialog
         dialog = CTK.DialogProxyLazy (URL_SAVE, {'title': _(SAVED_NOTICE), 'autoOpen': False, 'draggable': False, 'width': 500})
         CTK.publish (URL_SAVE, Save, dialog=dialog)
@@ -142,11 +163,11 @@ class Base (CTK.Page):
         CTK.Page.__init__ (self, template, heads, **kwargs)
 
         # Add the 'Save' dialog
-        js = SAVE_BUTTON %(dialog.JS_to_show())
+        js = SAVE_BUTTON %({'show_dialog_js': dialog.JS_to_show()})
         if CTK.cfg.has_changed():
             js += ".removeClass('saved');"
         else:
-            js += ';'
+            js += ".addClass('saved');"
 
         self += dialog
         self += CTK.RawHTML (js=js)
