@@ -1,16 +1,20 @@
-#!/usr/bin/env python
+#!/usr/local/bin/python2.7
 
 # Cherokee CLI configuration conversion
 #
 # Authors:
 #      Alvaro Lopez Ortega <alvaro@alobbs.com>
 #
-# Copyright (C) 2001-2009 Alvaro Lopez Ortega
+# Copyright (C) 2001-2010 Alvaro Lopez Ortega
 # This file is distributed under the GPL2 license.
 
+import os
 import sys
 
-from config import *
+# Import CTK
+sys.path.append (os.path.abspath (os.path.realpath(__file__) + '/../CTK/CTK'))
+from Config import *
+
 from configured import *
 from config_version import *
 
