@@ -1,6 +1,17 @@
+import CTK
 import string
 import os.path
-from util import split_list
+import re
+from util import split_list, get_real_path
+
+# Conditional Check
+# -----------------
+# By default the validations are not performed if there is no value to
+# check. That usually means that the configuation entry is being
+# removed. In case the entry should always be checked, the function
+# could define a property 'CHECK_ON_NO_VALUE' to enforce it.
+
+OPTIONAL = 'optional'
 
 def is_number (value):
     try:
@@ -31,13 +42,27 @@ def is_tcp_port (value):
     except:
         raise ValueError, _('Port must be a number')
     if tmp < 0 or tmp > 0xFFFF:
-        raise ValueError, _('Out of the range (1 to 65535)')
+        raise ValueError, _('Out of range (1 to 65535)')
+    return value
+
+def is_new_tcp_port (value):
+    value    = is_tcp_port (value)
+    bindings = CTK.cfg.keys ('server!bind')
+    ports    = [CTK.cfg.get_val('server!bind!%s!port'%(x)) for x in bindings]
+    if value in ports:
+        raise ValueError, _('Port already in use')
     return value
 
 def is_extension (value):
     is_not_empty(value)
     return value
 
+def is_web_path (value):
+    is_not_empty(value)
+    if value[0] == '/':
+        return value
+    raise ValueError, _('Malformed path')
+
 def is_path (value):
     is_not_empty(value)
     if value[0] == '/':
@@ -52,7 +77,7 @@ def is_list (value):
         return ''
     return ','.join(tmp)
 
-def is_dir_formated (value):
+def is_dir_formatted (value):
     is_path (value)
 
     try:
@@ -78,6 +103,12 @@ def is_exec_path (value):
         raise ValueError, _('It is not executable')
     return value
 
+def is_exec_file (value):
+    value = is_exec_path (value)
+    if not os.path.isfile(value):
+        raise ValueError, _('Path is not a regular file')
+    return value
+
 def is_extension_list (value):
     re = []
     for p in split_list(value):
@@ -127,14 +158,12 @@ def is_ipv6 (value):
         raise ValueError, _('Malformed IPv6')
     return value
 
-def is_local_dir_exists (value, cfg, nochroot=False):
-    value = is_path (value)
+def is_chroot_dir_exists (value):
+    return is_local_dir_exists (value, nochroot=True)
 
-    chroot = cfg.get_val('server!chroot')
-    if chroot and not nochroot:
-        path = os.path.normpath (chroot + os.path.sep + value)
-    else:
-        path = value
+def is_local_dir_exists (value, nochroot=False):
+    value = is_path (value)
+    path  = get_real_path (value, nochroot)
 
     if not os.path.exists(path):
         raise ValueError, _('Path does not exist')
@@ -144,14 +173,9 @@ def is_local_dir_exists (value, cfg, nochroot=False):
 
     return value
 
-def is_local_file_exists (value, cfg, nochroot=False):
+def is_local_file_exists (value, nochroot=False):
     value = is_path (value)
-
-    chroot = cfg.get_val('server!chroot')
-    if chroot and not nochroot:
-        path = os.path.normpath (chroot + os.path.sep + value)
-    else:
-        path = value
+    path  = get_real_path (value, nochroot)
 
     if not os.path.exists(path):
         raise ValueError, _('Path does not exist')
@@ -161,11 +185,11 @@ def is_local_file_exists (value, cfg, nochroot=False):
 
     return value
 
-def parent_is_dir (value, cfg, nochroot=False):
+def parent_is_dir (value, nochroot=False):
     value = is_path (value)
 
     dirname, filename = os.path.split(value)
-    is_local_dir_exists (dirname, cfg, nochroot)
+    is_local_dir_exists (dirname, nochroot)
 
     return value
 
@@ -182,6 +206,9 @@ def is_safe_id_list (value):
         ids.append(is_safe_id (id))
     return ','.join(ids)
 
+def is_header_name (value):
+    return is_safe_id (value)
+
 def int2bin(n, count=24):
     """returns the binary of integer n, using count number of digits"""
     return "".join([str((n >> y) & 1) for y in range(count-1, -1, -1)])
@@ -258,6 +285,7 @@ def is_not_empty (value):
     if len(value) <= 0:
         raise ValueError, _('Cannot be empty')
     return value
+is_not_empty.CHECK_ON_NO_VALUE = True
 
 def debug_fail (value):
     raise ValueError, _('Forced failure')
@@ -280,10 +308,10 @@ def is_url_or_path (value):
 
     raise ValueError, 'Not a URL, nor a path'
 
-def is_dev_null_or_local_dir_exists (value, cfg, nochroot=False):
+def is_dev_null_or_local_dir_exists (value, nochroot=False):
     if value == '/dev/null':
         return value
-    return is_local_dir_exists (value, cfg, nochroot)
+    return is_local_dir_exists (value, nochroot)
 
 def is_information_source (value):
     # /unix/path
@@ -312,10 +340,81 @@ def is_time (value):
 
     return value
 
-def is_new_host (value, cfg, nochroot):
-    for h in cfg.keys('vserver'):
-        if value == cfg.get_val('vserver!%s!nick'%(h)):
-            raise ValueError, _('Virtual host nick is already being used.')
+def is_new_vserver_nick (value):
+    value = is_not_empty(value)
+    for h in CTK.cfg.keys('vserver'):
+        if value == CTK.cfg.get_val('vserver!%s!nick'%(h)):
+            raise ValueError, _('Virtual Server nick name is already being used.')
     return value
 
 
+def is_safe_mime_type (mime):
+    mimes = CTK.cfg.keys ('mime')
+    if mime in mimes:
+        raise ValueError, _('Already in use')
+    return mime
+
+
+def is_safe_information_source_host (value, exclude = None):
+    host  = is_information_source (value)
+    keys  = ['source!%s!host'%(key) for key in CTK.cfg.keys ('source')]
+    hosts = [CTK.cfg.get_val(key) for key in keys if key != exclude]
+
+    if host in hosts:
+        raise ValueError, _('Already in use')
+    return host
+
+
+def is_safe_information_source_nick (value, exclude = None):
+    nick  = is_not_empty(value)
+    keys  = ['source!%s!nick'%(key) for key in CTK.cfg.keys ('source')]
+    nicks = [CTK.cfg.get_val(key) for key in keys if key != exclude]
+
+    if nick in nicks:
+        raise ValueError, _('Already in use')
+    return nick
+
+
+def is_safe_cfgval (key, cfg_str, new, safe):
+    new  = list(set(split_list (new))) # remove list duplicates
+    keys = CTK.cfg.keys (key)
+    cfg  = [CTK.cfg.get_val(cfg_str % key) for key in keys]
+    old  = [x for sublist in map(split_list, cfg) for x in sublist]
+
+    old.sort()
+    dupes = list (set(old) & set(new))
+    if safe: # Do not worry about safe values
+        safe  = split_list (safe)
+        dupes = list (set(dupes) - set(safe))
+
+    if dupes:
+        raise ValueError, '%s: %s' %(_('Already in use'), ', '.join(dupes))
+
+    return ','.join(new)
+
+
+def is_safe_mime_exts (new, safe = None):
+    return is_safe_cfgval ('mime', 'mime!%s!extensions', new, safe)
+
+
+def is_safe_icons_suffix (new, safe = None):
+    return is_safe_cfgval ('icons!suffix', 'icons!suffix!%s', new, safe)
+
+
+def is_safe_icons_file (new, safe = None):
+    return is_safe_cfgval ('icons!file', 'icons!file!%s', new, safe)
+
+def is_positive_int_4_multiple (value):
+    tmp = is_positive_int (value)
+    num = int(tmp)
+    while num % 4 != 0:
+        num += 1
+    return str(num)
+
+def is_email (value):
+    is_not_empty(value)
+
+    if re.match (r"^\S+@\S+\.\S+$", value) == None:
+        raise ValueError, _("Please insert a valid email")
+
+    return value
